require 'spec_helper'

describe Bankster::Representers::TransactionRepresenter do
  describe 'creation of json from multiple transactions' do
    let(:transaction_1) { FactoryGirl.build(:transaction, amount: Money.eur(123)) }
    let(:transaction_2) { FactoryGirl.build(:transaction, amount: Money.eur(123)) }
    let(:transactions) { [transaction_1, transaction_2] }
    subject { described_class.for_collection.prepare(transactions) }

    it 'creates json that matches the schema' do
      expect(subject.to_json).to match_json_schema(:transactions_schema, strict: true)
    end
  end

  describe 'creation of multiple bankster transactions from a collection api response' do
    let(:transaction_1) { FactoryGirl.build(:transaction, amount: Money.eur(123)) }
    let(:transaction_2) { FactoryGirl.build(:transaction, amount: Money.eur(345)) }
    let(:transactions) { [transaction_1, transaction_2] }
    let(:api_response) { described_class.for_collection.prepare(transactions).to_json }

    it 'parses collections and returns transaction objects' do
      parsed_transactions = [].extend(Bankster::Representers::TransactionRepresenter.for_collection).from_json(api_response)
      expect(parsed_transactions.first).to be_a(Bankster::Transaction)
      expect(parsed_transactions.last).to be_a(Bankster::Transaction)
      expect(parsed_transactions.count).to eql(2)
      expect(parsed_transactions.first.amount).to eql(transaction_1.amount)
      expect(parsed_transactions.last.amount).to eql(transaction_2.amount)
    end
  end

  describe 'creation of json from a Bankster::Transaction' do
    let(:transaction) { FactoryGirl.build(:transaction, amount: Money.eur(123)) }
    subject { transaction.extend(described_class) }

    it 'creates json that matches the schema' do
      expect(subject.to_json).to match_json_schema(:transaction_schema, strict: true)
    end

    describe 'attributes' do
      let(:json) { JSON.parse(subject.to_json) }

      it 'represents the amount as a hash with cents and currency' do
        expect(json['amount']['cents']).to eql("123")
        expect(json['amount']['currency']).to eql("EUR")
      end

      it 'represents the purpose' do
        expect(json['purpose']).to eql(transaction.purpose)
      end

      it 'represents the iban' do
        expect(json['iban']).to eql(transaction.iban)
      end

      it 'represents the bic' do
        expect(json['bic']).to eql(transaction.bic)
      end

      it 'represents the name' do
        expect(json['name']).to eql(transaction.name)
      end

      it 'represents the type' do
        expect(json['type']).to eql(transaction.type.to_s)
      end

      it 'represents the booking date as a iso 8609 string' do
        expect(json['booking_date']).to eql(Date.today.iso8601)
      end

      it 'represents the valutation date as a iso 8609 string' do
        expect(json['valutation_date']).to eql(Date.today.iso8601)
      end
    end
  end

  describe 'creation of a Bankster::Transaction from json' do
    let(:api_response) { FactoryGirl.build(:transaction_api_response) }
    let(:subject) { Bankster::Transaction.new.extend(described_class) }

    before do
      subject.from_json(api_response.to_json)
    end

    describe 'attributes' do
      it 'converts the money hash with cents and currency to a Money object' do
        expect(subject.amount).to eql(
          Money.new(
            api_response['amount']['cents'],
            api_response['amount']['currency']
          )
        )
      end

      it 'represents the purpose' do
        expect(subject.purpose).to eql(api_response['purpose'])
      end

      it 'represents the name' do
        expect(subject.name).to eql(api_response['name'])
      end

      it 'represents the iban' do
        expect(subject.iban).to eql(api_response['iban'])
      end

      it 'represents the bic' do
        expect(subject.bic).to eql(api_response['bic'])
      end

      it 'converts the type from a string to a symbol' do
        expect(subject.type).to eql(api_response['type'].to_sym)
      end

      it 'converts the booking date from a iso 8609 string to a date object' do
        expect(subject.booking_date).to eql(Date.parse(api_response['booking_date']))
      end

      it 'converts the valutation date from a iso 8609 string to a date object' do
        expect(subject.valutation_date).to eql(Date.parse(api_response['valutation_date']))
      end
    end
  end
end
